@@ -19,14 +19,14 @@
 */
 
 #include "module.h"
-#include "signals.h"
-#include "misc.h"
-#include "special-vars.h"
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/misc.h>
+#include <irssi/src/core/special-vars.h>
 
-#include "servers.h"
-#include "channels.h"
-#include "channels-setup.h"
-#include "nicklist.h"
+#include <irssi/src/core/servers.h>
+#include <irssi/src/core/channels.h>
+#include <irssi/src/core/channels-setup.h>
+#include <irssi/src/core/nicklist.h>
 
 GSList *channels; /* List of all channels */
 
@@ -126,9 +126,8 @@ CHANNEL_REC *channel_find(SERVER_REC *server, const char *name)
 		return channel_find_server(server, name);
 
 	/* find from any server */
-	return gslist_foreach_find(servers,
-				   (FOREACH_FIND_FUNC) channel_find_server,
-				   (void *) name);
+	return i_slist_foreach_find(servers, (FOREACH_FIND_FUNC) channel_find_server,
+	                            (void *) name);
 }
 
 void channel_change_name(CHANNEL_REC *channel, const char *name)
@@ -153,9 +152,8 @@ void channel_change_visible_name(CHANNEL_REC *channel, const char *name)
 
 static CHANNEL_REC *channel_find_servers(GSList *servers, const char *name)
 {
-	return gslist_foreach_find(servers,
-				   (FOREACH_FIND_FUNC) channel_find_server,
-				   (void *) name);
+	return i_slist_foreach_find(servers, (FOREACH_FIND_FUNC) channel_find_server,
+	                            (void *) name);
 }
 
 static GSList *servers_find_chatnet_except(SERVER_REC *server)
