@@ -19,17 +19,17 @@
 */
 
 #include "module.h"
-#include "signals.h"
-#include "commands.h"
-#include "misc.h"
-#include "special-vars.h"
-#include "window-item-def.h"
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/commands.h>
+#include <irssi/src/core/misc.h>
+#include <irssi/src/core/special-vars.h>
+#include <irssi/src/core/window-item-def.h>
 
-#include "servers.h"
-#include "channels.h"
+#include <irssi/src/core/servers.h>
+#include <irssi/src/core/channels.h>
 
-#include "lib-config/iconfig.h"
-#include "settings.h"
+#include <irssi/src/lib-config/iconfig.h>
+#include <irssi/src/core/settings.h>
 
 GSList *commands;
 char *current_command;
@@ -339,11 +339,28 @@ void command_runsub(const char *cmd, const char *data,
 	g_free(orig);
 }
 
+static char *optname(char *option)
+{
+	char *opt = option;
+	if (*opt == '~')
+		opt++;
+	if (iscmdtype(*opt))
+		opt++;
+	return opt;
+}
+
+static gboolean optflag(char *option, char *flag)
+{
+	if (*option == '~')
+		return optflag(option + 1, flag);
+
+	return (strchr(flag, *option) != NULL) || (!iscmdtype(*option) && strchr(flag, ' '));
+}
+
 static GSList *optlist_find(GSList *optlist, const char *option)
 {
 	while (optlist != NULL) {
-		char *name = optlist->data;
-		if (iscmdtype(*name)) name++;
+		char *name = optname(optlist->data);
 
 		if (g_ascii_strcasecmp(name, option) == 0)
 			return optlist;
@@ -369,7 +386,7 @@ int command_have_option(const char *cmd, const char *option)
 		return FALSE;
 
 	for (tmp = rec->options; *tmp != NULL; tmp++) {
-		char *name = iscmdtype(**tmp) ? (*tmp)+1 : *tmp;
+		char *name = optname(*tmp);
 
 		if (g_ascii_strcasecmp(name, option) == 0)
 			return TRUE;
@@ -399,7 +416,7 @@ static void command_calc_options(COMMAND_REC *rec, const char *options)
 
 	/* merge the options */
 	for (tmp = optlist; *tmp != NULL; tmp++) {
-		name = iscmdtype(**tmp) ? (*tmp)+1 : *tmp;
+		name = optname(*tmp);
 
 		oldopt = optlist_find(list, name);
 		if (oldopt != NULL) {
@@ -414,7 +431,7 @@ static void command_calc_options(COMMAND_REC *rec, const char *options)
 	g_strfreev(optlist);
 
 	/* linked list -> string[] */
-	str = gslist_to_string(list, " ");
+	str = i_slist_to_string(list, " ");
 	rec->options = g_strsplit(str, " ", -1);
         g_free(str);
 
@@ -501,7 +518,7 @@ char *cmd_get_quoted_param(char **data)
 	while (**data != '\0' && (**data != quote ||
 				  ((*data)[1] != ' ' && (*data)[1] != '\0'))) {
 		if (**data == '\\' && (*data)[1] != '\0')
-                        g_memmove(*data, (*data)+1, strlen(*data));
+                        memmove(*data, (*data)+1, strlen(*data));
 		(*data)++;
 	}
 
@@ -529,7 +546,7 @@ static int option_find(char **array, const char *option)
 
 	found = -1; index = 0; multiple = FALSE;
 	for (tmp = array; *tmp != NULL; tmp++, index++) {
-		const char *text = *tmp + iscmdtype(**tmp);
+		const char *text = optname(*tmp);
 
 		if (g_ascii_strncasecmp(text, option, len) == 0) {
 			if (text[len] == '\0') {
@@ -568,9 +585,9 @@ static int get_cmd_options(char **data, int ignore_unknown,
 	option = NULL; pos = -1;
 	for (;;) {
 		if (**data == '\0' || **data == '-') {
-			if (option != NULL && *optlist[pos] == '+') {
+			if (option != NULL && optflag(optlist[pos], "+")) {
 				/* required argument missing! */
-                                *data = optlist[pos] + 1;
+				*data = optname(optlist[pos]);
 				return CMDERR_OPTION_ARG_MISSING;
 			}
 		}
@@ -621,14 +638,12 @@ static int get_cmd_options(char **data, int ignore_unknown,
 			if (pos >= 0) {
 				/* if we used a shortcut of parameter, put
 				   the whole parameter name in options table */
-				option = optlist[pos] +
-					iscmdtype(*optlist[pos]);
+				option = optname(optlist[pos]);
 			}
 			if (options != NULL && pos != -3)
 				g_hash_table_insert(options, option, "");
 
-			if (pos < 0 || !iscmdtype(*optlist[pos]) ||
-			    *optlist[pos] == '!')
+			if (pos < 0 || optflag(optlist[pos], " !"))
 				option = NULL;
 
 			while (**data == ' ') (*data)++;
@@ -638,7 +653,7 @@ static int get_cmd_options(char **data, int ignore_unknown,
 		if (option == NULL)
 			break;
 
-		if (*optlist[pos] == '@' && !is_numeric(*data, ' '))
+		if (optflag(optlist[pos], "@") && !is_numeric(*data, ' '))
 			break; /* expected a numeric argument */
 
 		/* save the argument */
@@ -725,8 +740,7 @@ int cmd_get_params(const char *data, gpointer *free_me, int count, ...)
 		opthash = (GHashTable **) va_arg(args, GHashTable **);
 
 		rec->options = *opthash =
-			g_hash_table_new((GHashFunc) g_istr_hash,
-					 (GCompareFunc) g_istr_equal);
+		    g_hash_table_new((GHashFunc) i_istr_hash, (GCompareFunc) i_istr_equal);
 
 		ignore_unknown = count & PARAM_FLAG_UNKNOWN_OPTIONS;
 		error = get_cmd_options(&datad, ignore_unknown,
@@ -817,7 +831,7 @@ void commands_remove_module(const char *module)
 		COMMAND_REC *rec = tmp->data;
 
                 next = tmp->next;
-		modlist = gslist_find_string(rec->modules, module);
+		modlist = i_slist_find_string(rec->modules, module);
 		if (modlist != NULL)
 			command_module_unbind_all(rec, modlist->data);
 	}
@@ -851,8 +865,7 @@ static int cmd_protocol_match(COMMAND_REC *cmd, SERVER_REC *server)
 #define alias_runstack_pop(alias) \
 	alias_runstack = g_slist_remove(alias_runstack, alias)
 
-#define alias_runstack_find(alias) \
-        (gslist_find_icase_string(alias_runstack, alias) != NULL)
+#define alias_runstack_find(alias) (i_slist_find_icase_string(alias_runstack, alias) != NULL)
 
 static void parse_command(const char *command, int expand_aliases,
 			  SERVER_REC *server, void *item)
