@@ -19,20 +19,20 @@
 */
 
 #include "module.h"
-#include "signals.h"
-#include "misc.h"
-#include "levels.h"
-#include "lib-config/iconfig.h"
-#include "settings.h"
-#include "iregex.h"
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/misc.h>
+#include <irssi/src/core/levels.h>
+#include <irssi/src/lib-config/iconfig.h>
+#include <irssi/src/core/settings.h>
+#include <irssi/src/core/iregex.h>
 
-#include "masks.h"
-#include "servers.h"
-#include "channels.h"
-#include "nicklist.h"
-#include "nickmatch-cache.h"
+#include <irssi/src/core/masks.h>
+#include <irssi/src/core/servers.h>
+#include <irssi/src/core/channels.h>
+#include <irssi/src/core/nicklist.h>
+#include <irssi/src/core/nickmatch-cache.h>
 
-#include "ignore.h"
+#include <irssi/src/core/ignore.h>
 
 GSList *ignores;
 
@@ -81,7 +81,7 @@ static int ignore_match_pattern(IGNORE_REC *rec, const char *text)
  * used as a flag to indicate it should only look at ignore items with NO_ACT.
  * However we also want to allow NO_ACT combined with levels, so mask it out and
  * match levels if set. */
-#define FLAG_MSGLEVELS ( MSGLEVEL_NO_ACT | MSGLEVEL_HIDDEN )
+#define FLAG_MSGLEVELS (MSGLEVEL_NO_ACT | MSGLEVEL_HIDDEN | MSGLEVEL_NOHILIGHT)
 static int ignore_match_level(IGNORE_REC *rec, int level, int flags)
 {
 	level &= ~FLAG_MSGLEVELS;
@@ -214,6 +214,9 @@ int ignore_check_plus(SERVER_REC *server, const char *nick, const char *address,
 	if (ignore_check_flags(server, nick, address, target, msg, olevel, MSGLEVEL_HIDDEN))
 		*level |= MSGLEVEL_HIDDEN;
 
+	if (ignore_check_flags(server, nick, address, target, msg, olevel, MSGLEVEL_NOHILIGHT))
+		*level |= MSGLEVEL_NOHILIGHT;
+
 	return FALSE;
 }
 
@@ -252,6 +255,12 @@ IGNORE_REC *ignore_find_full(const char *servertag, const char *mask, const char
 		if (!(flags & IGNORE_FIND_HIDDEN) && (rec->level & MSGLEVEL_HIDDEN) != 0)
 			continue;
 
+		if ((flags & IGNORE_FIND_NOHILIGHT) && (rec->level & MSGLEVEL_NOHILIGHT) == 0)
+			continue;
+
+		if (!(flags & IGNORE_FIND_NOHILIGHT) && (rec->level & MSGLEVEL_NOHILIGHT) != 0)
+			continue;
+
 		if ((rec->mask == NULL && mask != NULL) ||
 		    (rec->mask != NULL && mask == NULL))
 			continue;
@@ -299,16 +308,6 @@ IGNORE_REC *ignore_find(const char *servertag, const char *mask, char **channels
 	return ignore_find_full(servertag, mask, NULL, channels, 0);
 }
 
-IGNORE_REC *ignore_find_noact(const char *servertag, const char *mask, char **channels, int noact)
-{
-	return ignore_find_full(servertag, mask, NULL, channels, IGNORE_FIND_NOACT);
-}
-
-IGNORE_REC *ignore_find_hidden(const char *servertag, const char *mask, char **channels, int hidden)
-{
-	return ignore_find_full(servertag, mask, NULL, channels, IGNORE_FIND_HIDDEN);
-}
-
 static void ignore_set_config(IGNORE_REC *rec)
 {
 	CONFIG_NODE *node;
@@ -491,6 +490,11 @@ static void read_ignores(void)
 	nickmatch_rebuild(nickmatch);
 }
 
+static void free_cache_matches(GSList *matches)
+{
+	g_slist_free(matches);
+}
+
 static void ignore_nick_cache(GHashTable *list, CHANNEL_REC *channel,
 			      NICK_REC *nick)
 {
@@ -521,7 +525,7 @@ static void ignore_nick_cache(GHashTable *list, CHANNEL_REC *channel,
 void ignore_init(void)
 {
 	ignores = NULL;
-	nickmatch = nickmatch_init(ignore_nick_cache);
+	nickmatch = nickmatch_init(ignore_nick_cache, (GDestroyNotify) free_cache_matches);
 	time_tag = g_timeout_add(1000, (GSourceFunc) unignore_timeout, NULL);
 
         read_ignores();
