@@ -1,8 +1,7 @@
-#ifndef __MISC_H
-#define __MISC_H
+#ifndef IRSSI_CORE_MISC_H
+#define IRSSI_CORE_MISC_H
 
-int g_input_add_poll(int fd, int priority, int condition,
-		     GInputFunction function, void *data);
+int i_input_add_poll(int fd, int priority, int condition, GInputFunction function, void *data);
 
 /* `str' should be type char[MAX_INT_STRLEN] */
 #define ltoa(str, num) \
@@ -11,27 +10,37 @@ int g_input_add_poll(int fd, int priority, int condition,
 typedef void* (*FOREACH_FIND_FUNC) (void *item, void *data);
 typedef int (*COLUMN_LEN_FUNC)(void *data);
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 /* Returns 1 if tv1 > tv2, -1 if tv2 > tv1 or 0 if they're equal. */
-int g_timeval_cmp(const GTimeVal *tv1, const GTimeVal *tv2);
+int g_timeval_cmp(const GTimeVal *tv1, const GTimeVal *tv2) G_GNUC_DEPRECATED;
 /* Returns "tv1 - tv2", returns the result in milliseconds. Note that
    if the difference is too large, the result might be invalid. */
-long get_timeval_diff(const GTimeVal *tv1, const GTimeVal *tv2);
+long get_timeval_diff(const GTimeVal *tv1, const GTimeVal *tv2) G_GNUC_DEPRECATED;
+#pragma GCC diagnostic pop
 
-GSList *gslist_find_string(GSList *list, const char *key);
-GSList *gslist_find_icase_string(GSList *list, const char *key);
-GList *glist_find_string(GList *list, const char *key);
-GList *glist_find_icase_string(GList *list, const char *key);
-GSList *gslist_remove_string (GSList *list, const char *str) G_GNUC_DEPRECATED;
-GSList *gslist_delete_string (GSList *list, const char *str, GDestroyNotify free_func);
+#if GLIB_CHECK_VERSION(2, 56, 0)
+/* nothing */
+#else
+/* compatibility code for old GLib */
+GDateTime *g_date_time_new_from_iso8601(const gchar *iso_date, GTimeZone *default_tz);
+#endif
 
-void gslist_free_full (GSList *list, GDestroyNotify free_func);
+GSList *i_slist_find_string(GSList *list, const char *key);
+GSList *i_slist_find_icase_string(GSList *list, const char *key);
+GList *i_list_find_string(GList *list, const char *key);
+GList *i_list_find_icase_string(GList *list, const char *key);
+GSList *i_slist_remove_string(GSList *list, const char *str) G_GNUC_DEPRECATED;
+GSList *i_slist_delete_string(GSList *list, const char *str, GDestroyNotify free_func);
 
-void *gslist_foreach_find(GSList *list, FOREACH_FIND_FUNC func, const void *data);
+void i_slist_free_full(GSList *list, GDestroyNotify free_func);
+
+void *i_slist_foreach_find(GSList *list, FOREACH_FIND_FUNC func, const void *data);
 
 /* `list' contains pointer to structure with a char* to string. */
 char *gslistptr_to_string(GSList *list, int offset, const char *delimiter);
 /* `list' contains char* */
-char *gslist_to_string(GSList *list, const char *delimiter);
+char *i_slist_to_string(GSList *list, const char *delimiter);
 
 GList *optlist_remove_known(const char *cmd, GHashTable *optlist);
 
@@ -39,18 +48,18 @@ GList *optlist_remove_known(const char *cmd, GHashTable *optlist);
 char *convert_home(const char *path);
 
 /* Case-insensitive string hash functions */
-int g_istr_equal(gconstpointer v, gconstpointer v2);
-unsigned int g_istr_hash(gconstpointer v);
+int i_istr_equal(gconstpointer v, gconstpointer v2);
+unsigned int i_istr_hash(gconstpointer v);
 
 /* Case-insensitive GCompareFunc func */
-int g_istr_cmp(gconstpointer v, gconstpointer v2);
+int i_istr_cmp(gconstpointer v, gconstpointer v2);
 
 /* Find `mask' from `data', you can use * and ? wildcards. */
 int match_wildcards(const char *mask, const char *data);
 
 /* octal <-> decimal conversions */
 int octal2dec(int octal);
-int dec2octal(int decimal);
+int dec2octal(int decimal) G_GNUC_DEPRECATED;
 
 /* Get time in human readable form with localtime() + asctime() */
 char *my_asctime(time_t t);
