@@ -1,5 +1,5 @@
-#ifndef __RECODE_H
-#define __RECODE_H
+#ifndef IRSSI_CORE_RECODE_H
+#define IRSSI_CORE_RECODE_H
 
 char *recode_in (const SERVER_REC *server, const char *str, const char *target);
 char *recode_out (const SERVER_REC *server, const char *str, const char *target);
@@ -12,4 +12,4 @@ void recode_update_charset(void);
 void recode_init (void);
 void recode_deinit (void);
 
-#endif /* __RECODE_H */
+#endif /* IRSSI_CORE_RECODE_H */
