@@ -19,21 +19,22 @@
 */
 
 #include "module.h"
-#include "signals.h"
-#include "commands.h"
-#include "net-disconnect.h"
-#include "net-nonblock.h"
-#include "net-sendbuffer.h"
-#include "misc.h"
-#include "rawlog.h"
-#include "settings.h"
+#include <irssi/src/core/commands.h>
+#include <irssi/src/core/misc.h>
+#include <irssi/src/core/net-disconnect.h>
+#include <irssi/src/core/net-nonblock.h>
+#include <irssi/src/core/net-sendbuffer.h>
+#include <irssi/src/core/rawlog.h>
+#include <irssi/src/core/refstrings.h>
+#include <irssi/src/core/settings.h>
+#include <irssi/src/core/signals.h>
 
-#include "chat-protocols.h"
-#include "servers.h"
-#include "servers-reconnect.h"
-#include "servers-setup.h"
-#include "channels.h"
-#include "queries.h"
+#include <irssi/src/core/chat-protocols.h>
+#include <irssi/src/core/servers.h>
+#include <irssi/src/core/servers-reconnect.h>
+#include <irssi/src/core/servers-setup.h>
+#include <irssi/src/core/channels.h>
+#include <irssi/src/core/queries.h>
 
 GSList *servers, *lookup_servers;
 
@@ -156,6 +157,7 @@ static void server_connect_callback_init(SERVER_REC *server, GIOChannel *handle)
 	error = net_geterror(handle);
 	if (error != 0) {
 		server->connection_lost = TRUE;
+		server->connrec->last_failed_family = server->connrec->chosen_family;
 		server_connect_failed(server, g_strerror(error));
 		return;
 	}
@@ -176,16 +178,16 @@ static void server_connect_callback_init_ssl(SERVER_REC *server, GIOChannel *han
 	error = irssi_ssl_handshake(handle);
 	if (error == -1) {
 		server->connection_lost = TRUE;
+		server->connrec->last_failed_family = server->connrec->chosen_family;
 		server_connect_failed(server, NULL);
 		return;
 	}
 	if (error & 1) {
 		if (server->connect_tag != -1)
 			g_source_remove(server->connect_tag);
-		server->connect_tag = g_input_add(handle, error == 1 ? G_INPUT_READ : G_INPUT_WRITE,
-						  (GInputFunction)
-						  server_connect_callback_init_ssl,
-						  server);
+		server->connect_tag =
+		    i_input_add(handle, error == 1 ? I_INPUT_READ : I_INPUT_WRITE,
+		                (GInputFunction) server_connect_callback_init_ssl, server);
 		return;
 	}
 
@@ -216,6 +218,7 @@ static void server_real_connect(SERVER_REC *server, IPADDR *ip,
 		return;
 
 	if (ip != NULL) {
+		server->connrec->chosen_family = ip->family;
 		own_ip = IPADDR_IS_V6(ip) ? server->connrec->own_ip6 : server->connrec->own_ip4;
 		port = server->connrec->proxy != NULL ?
 			server->connrec->proxy_port : server->connrec->port;
@@ -251,19 +254,21 @@ static void server_real_connect(SERVER_REC *server, IPADDR *ip,
 			server->no_reconnect = TRUE;
 
 		server->connection_lost = TRUE;
+		if (ip != NULL) {
+			server->connrec->last_failed_family = ip->family;
+		}
 		server_connect_failed(server, errmsg2 ? errmsg2 : errmsg);
 		g_free(errmsg2);
 	} else {
+		server->connrec->last_failed_family = 0;
 		if (!server->connrec->use_tls)
 			server->handle = net_sendbuffer_create(handle, 0);
 		if (server->connrec->use_tls)
 			server_connect_callback_init_ssl(server, handle);
 		else
-		server->connect_tag =
-			g_input_add(handle, G_INPUT_WRITE | G_INPUT_READ,
-				    (GInputFunction)
-				    server_connect_callback_init,
-				    server);
+			server->connect_tag =
+			    i_input_add(handle, I_INPUT_WRITE | I_INPUT_READ,
+			                (GInputFunction) server_connect_callback_init, server);
 	}
 }
 
@@ -272,7 +277,6 @@ static void server_connect_callback_readpipe(SERVER_REC *server)
 	RESOLVED_IP_REC iprec;
         IPADDR *ip;
 	const char *errormsg;
-	char *servername = NULL;
 
 	g_source_remove(server->connect_tag);
 	server->connect_tag = -1;
@@ -294,31 +298,27 @@ static void server_connect_callback_readpipe(SERVER_REC *server)
 	} else if (server->connrec->family == AF_INET) {
 		/* force IPv4 connection */
 		ip = iprec.ip4.family == 0 ? NULL : &iprec.ip4;
-		servername = iprec.host4;
 	} else if (server->connrec->family == AF_INET6) {
 		/* force IPv6 connection */
 		ip = iprec.ip6.family == 0 ? NULL : &iprec.ip6;
-		servername = iprec.host6;
 	} else {
-		/* pick the one that was found, or if both do it like
-		   /SET resolve_prefer_ipv6 says. */
+		/* pick the one that was found. if both were found:
+		   1. disprefer the last one that failed
+		   2. prefer ipv4 over ipv6 unless resolve_prefer_ipv6 is set
+		*/
 		if (iprec.ip4.family == 0 ||
 		    (iprec.ip6.family != 0 &&
-		     settings_get_bool("resolve_prefer_ipv6"))) {
+		     (server->connrec->last_failed_family == AF_INET ||
+		      (settings_get_bool("resolve_prefer_ipv6") &&
+		       server->connrec->last_failed_family != AF_INET6)))) {
 			ip = &iprec.ip6;
-			servername = iprec.host6;
 		} else {
 			ip = &iprec.ip4;
-			servername = iprec.host4;
 		}
 	}
 
 	if (ip != NULL) {
 		/* host lookup ok */
-		if (servername) {
-			g_free(server->connrec->address);
-			server->connrec->address = g_strdup(servername);
-		}
 		server_real_connect(server, ip, NULL);
 		errormsg = NULL;
 	} else {
@@ -344,8 +344,6 @@ static void server_connect_callback_readpipe(SERVER_REC *server)
 	}
 
 	g_free(iprec.errorstr);
-	g_free(iprec.host4);
-	g_free(iprec.host6);
 }
 
 SERVER_REC *server_connect(SERVER_CONNECT_REC *conn)
@@ -370,6 +368,9 @@ void server_connect_init(SERVER_REC *server)
 	MODULE_DATA_INIT(server);
 	server->type = module_get_uniq_id("SERVER", 0);
 	server_ref(server);
+	server->current_incoming_meta =
+	    g_hash_table_new_full(g_str_hash, (GEqualFunc) g_str_equal,
+	                          (GDestroyNotify) i_refstr_release, (GDestroyNotify) g_free);
 
 	server->nick = g_strdup(server->connrec->nick);
 	if (server->connrec->username == NULL || *server->connrec->username == '\0') {
@@ -422,20 +423,17 @@ int server_start_connect(SERVER_REC *server)
 			return FALSE;
 		}
 
-		server->connect_pipe[0] = g_io_channel_new(fd[0]);
-		server->connect_pipe[1] = g_io_channel_new(fd[1]);
+		server->connect_pipe[0] = i_io_channel_new(fd[0]);
+		server->connect_pipe[1] = i_io_channel_new(fd[1]);
 
 		connect_address = server->connrec->proxy != NULL ?
 			server->connrec->proxy : server->connrec->address;
 		server->connect_pid =
 			net_gethostbyname_nonblock(connect_address,
-						   server->connect_pipe[1],
-						   settings_get_bool("resolve_reverse_lookup"));
+						   server->connect_pipe[1], 0);
 		server->connect_tag =
-			g_input_add(server->connect_pipe[0], G_INPUT_READ,
-				    (GInputFunction)
-				    server_connect_callback_readpipe,
-				    server);
+		    i_input_add(server->connect_pipe[0], I_INPUT_READ,
+		                (GInputFunction) server_connect_callback_readpipe, server);
 
 		server->connect_time = time(NULL);
 		lookup_servers = g_slist_append(lookup_servers, server);
@@ -554,6 +552,7 @@ int server_unref(SERVER_REC *server)
 	g_free(server->away_reason);
 	g_free(server->nick);
 	g_free(server->tag);
+	g_hash_table_destroy(server->current_incoming_meta);
 
 	server->type = 0;
 	g_free(server);
@@ -674,6 +673,22 @@ void server_change_nick(SERVER_REC *server, const char *nick)
 	signal_emit("server nick changed", 1, server);
 }
 
+void server_meta_stash(SERVER_REC *server, const char *meta_key, const char *meta_value)
+{
+	g_hash_table_replace(server->current_incoming_meta, i_refstr_intern(meta_key),
+	                     g_strdup(meta_value));
+}
+
+const char *server_meta_stash_find(SERVER_REC *server, const char *meta_key)
+{
+	return g_hash_table_lookup(server->current_incoming_meta, meta_key);
+}
+
+void server_meta_clear_all(SERVER_REC *server)
+{
+	g_hash_table_remove_all(server->current_incoming_meta);
+}
+
 /* Update own IPv4 and IPv6 records */
 void server_connect_own_ip_save(SERVER_CONNECT_REC *conn,
 				IPADDR *ip4, IPADDR *ip6)
@@ -750,7 +765,6 @@ static void sig_chat_protocol_deinit(CHAT_PROTOCOL_REC *proto)
 void servers_init(void)
 {
 	settings_add_bool("server", "resolve_prefer_ipv6", FALSE);
-	settings_add_bool("server", "resolve_reverse_lookup", FALSE);
 	lookup_servers = servers = NULL;
 
 	signal_add("chat protocol deinit", (SIGNAL_FUNC) sig_chat_protocol_deinit);
