@@ -19,10 +19,10 @@
 */
 
 #include "module.h"
-#include "signals.h"
-#include "commands.h"
-#include "settings.h"
-#include "write-buffer.h"
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/commands.h>
+#include <irssi/src/core/settings.h>
+#include <irssi/src/core/write-buffer.h>
 
 #define BUFFER_BLOCK_SIZE 2048
 
