@@ -19,16 +19,16 @@
 */
 
 #include "module.h"
-#include "module-formats.h"
-#include "signals.h"
-#include "commands.h"
-#include "levels.h"
-#include "misc.h"
-#include "lib-config/iconfig.h"
-#include "settings.h"
+#include <irssi/src/fe-common/core/module-formats.h>
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/commands.h>
+#include <irssi/src/core/levels.h>
+#include <irssi/src/core/misc.h>
+#include <irssi/src/lib-config/iconfig.h>
+#include <irssi/src/core/settings.h>
 
-#include "completion.h"
-#include "printtext.h"
+#include <irssi/src/fe-common/core/completion.h>
+#include <irssi/src/fe-common/core/printtext.h>
 
 static GList *complist; /* list of commands we're currently completing */
 static char *last_line;
@@ -104,7 +104,7 @@ char *auto_word_complete(const char *line, int *pos)
 
 	/* check for words in autocompletion list */
 	replace = completion_find(word, TRUE);
-	if (replace == NULL) {
+	if (replace == NULL || (!g_strcmp0(replace, word))) {
 		ret = NULL;
 		g_string_free(result, TRUE);
 	} else {
@@ -425,7 +425,8 @@ static GList *completion_get_settings(const char *key, SettingType type)
 		SETTINGS_REC *rec = tmp->data;
 
 		if ((type == SETTING_TYPE_ANY || rec->type == type) && g_ascii_strncasecmp(rec->key, key, len) == 0)
-			complist = g_list_insert_sorted(complist, g_strdup(rec->key), (GCompareFunc) g_istr_cmp);
+			complist = g_list_insert_sorted(complist, g_strdup(rec->key),
+			                                (GCompareFunc) i_istr_cmp);
 	}
 	g_slist_free(sets);
 	return complist;
@@ -459,8 +460,9 @@ static GList *completion_get_aliases(const char *alias, char cmdchar)
 			/* add matching alias to completion list, aliases will
 			   be appended after command completions and kept in
 			   uppercase to show it's an alias */
-			if (glist_find_icase_string(complist, word) == NULL)
-				complist = g_list_insert_sorted(complist, word, (GCompareFunc) g_istr_cmp);
+			if (i_list_find_icase_string(complist, word) == NULL)
+				complist =
+				    g_list_insert_sorted(complist, word, (GCompareFunc) i_istr_cmp);
 			else
 				g_free(word);
 		}
@@ -488,8 +490,9 @@ static GList *completion_get_commands(const char *cmd, char cmdchar)
 		if (g_ascii_strncasecmp(rec->cmd, cmd, len) == 0) {
 			word = cmdchar == '\0' ? g_strdup(rec->cmd) :
 				g_strdup_printf("%c%s", cmdchar, rec->cmd);
-			if (glist_find_icase_string(complist, word) == NULL)
-				complist = g_list_insert_sorted(complist, word, (GCompareFunc) g_istr_cmp);
+			if (i_list_find_icase_string(complist, word) == NULL)
+				complist =
+				    g_list_insert_sorted(complist, word, (GCompareFunc) i_istr_cmp);
 			else
 				g_free(word);
 		}
@@ -523,7 +526,8 @@ static GList *completion_get_subcommands(const char *cmd)
 			continue;
 
 		if (g_ascii_strncasecmp(rec->cmd, cmd, len) == 0)
-			complist = g_list_insert_sorted(complist, g_strdup(rec->cmd+skip), (GCompareFunc) g_istr_cmp);
+			complist = g_list_insert_sorted(complist, g_strdup(rec->cmd + skip),
+			                                (GCompareFunc) i_istr_cmp);
 	}
 	return complist;
 }
@@ -544,7 +548,11 @@ static GList *completion_get_options(const char *cmd, const char *option)
 	list = NULL;
 	len = strlen(option);
 	for (tmp = rec->options; *tmp != NULL; tmp++) {
-		const char *optname = *tmp + iscmdtype(**tmp);
+		const char *optname;
+		if (**tmp == '~')
+			continue; /* deprecated or hidden option */
+
+		optname = *tmp + iscmdtype(**tmp);
 
 		if (len == 0 || g_ascii_strncasecmp(optname, option, len) == 0)
                         list = g_list_append(list, g_strconcat("-", optname, NULL));
