@@ -19,24 +19,24 @@
 */
 
 #include "module.h"
-#include "modules.h"
-#include "signals.h"
-#include "commands.h"
-#include "pidwait.h"
-#include "line-split.h"
-#include "network.h"
-#include "net-sendbuffer.h"
-#include "misc.h"
-#include "levels.h"
+#include <irssi/src/core/modules.h>
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/commands.h>
+#include <irssi/src/core/pidwait.h>
+#include <irssi/src/core/line-split.h>
+#include <irssi/src/core/network.h>
+#include <irssi/src/core/net-sendbuffer.h>
+#include <irssi/src/core/misc.h>
+#include <irssi/src/core/levels.h>
 
-#include "servers.h"
-#include "channels.h"
-#include "queries.h"
+#include <irssi/src/core/servers.h>
+#include <irssi/src/core/channels.h>
+#include <irssi/src/core/queries.h>
 
-#include "printtext.h"
-#include "fe-exec.h"
-#include "fe-windows.h"
-#include "window-items.h"
+#include <irssi/src/fe-common/core/printtext.h>
+#include <irssi/src/fe-common/core/fe-exec.h>
+#include <irssi/src/fe-common/core/fe-windows.h>
+#include <irssi/src/fe-common/core/window-items.h>
 
 #include <signal.h>
 #include <stdlib.h>
@@ -281,8 +281,8 @@ static void exec_show_list(void)
 
 static void process_exec(PROCESS_REC *rec, const char *cmd)
 {
-	const char *shell_args[4] = { "/bin/sh", "-c", NULL, NULL };
-        char **args;
+	const char *shell_args[4] = { FHS_PREFIX "/bin/sh", "-c", NULL, NULL };
+	char **args;
 	int in[2], out[2];
         int n;
 
@@ -302,9 +302,9 @@ static void process_exec(PROCESS_REC *rec, const char *cmd)
 
 	if (rec->pid != 0) {
 		/* parent process */
-                GIOChannel *outio = g_io_channel_new(in[1]);
+		GIOChannel *outio = i_io_channel_new(in[1]);
 
-		rec->in = g_io_channel_new(out[0]);
+		rec->in = i_io_channel_new(out[0]);
 		rec->out = net_sendbuffer_create(outio, 0);
 
                 close(out[1]);
@@ -316,11 +316,13 @@ static void process_exec(PROCESS_REC *rec, const char *cmd)
 	/* child process, try to clean up everything */
 	setsid();
 
+#ifndef __ANDROID__
 	if (setuid(getuid()) != 0)
 		_exit(EXIT_FAILURE);
 
 	if (setgid(getgid()) != 0)
 		_exit(EXIT_FAILURE);
+#endif
 
 	signal(SIGINT, SIG_IGN);
 	signal(SIGQUIT, SIG_DFL);
@@ -339,7 +341,7 @@ static void process_exec(PROCESS_REC *rec, const char *cmd)
 	if (rec->shell) {
 		execvp(shell_args[0], (char **) shell_args);
 
-		fprintf(stderr, "Exec: /bin/sh: %s\n", g_strerror(errno));
+		fprintf(stderr, "Exec: " FHS_PREFIX "/bin/sh: %s\n", g_strerror(errno));
 	} else {
 		args = g_strsplit(cmd, " ", -1);
                 execvp(args[0], args);
@@ -522,9 +524,8 @@ static void handle_exec(const char *args, GHashTable *optlist,
 	level = g_hash_table_lookup(optlist, "level");
 	rec->level = level == NULL ? MSGLEVEL_CLIENTCRAP : level2bits(level, NULL);
 
-	rec->read_tag = g_input_add(rec->in, G_INPUT_READ,
-				    (GInputFunction) sig_exec_input_reader,
-				    rec);
+	rec->read_tag =
+	    i_input_add(rec->in, I_INPUT_READ, (GInputFunction) sig_exec_input_reader, rec);
 	processes = g_slist_append(processes, rec);
 
 	if (rec->target == NULL && interactive)
