@@ -19,19 +19,19 @@
 */
 
 #include "module.h"
-#include "module-formats.h"
-#include "modules.h"
-#include "signals.h"
-#include "commands.h"
-#include "servers.h"
-#include "misc.h"
-#include "settings.h"
+#include <irssi/src/fe-common/core/module-formats.h>
+#include <irssi/src/core/modules.h>
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/commands.h>
+#include <irssi/src/core/servers.h>
+#include <irssi/src/core/misc.h>
+#include <irssi/src/core/settings.h>
 
-#include "levels.h"
+#include <irssi/src/core/levels.h>
 
-#include "printtext.h"
-#include "fe-windows.h"
-#include "window-items.h"
+#include <irssi/src/fe-common/core/printtext.h>
+#include <irssi/src/fe-common/core/fe-windows.h>
+#include <irssi/src/fe-common/core/window-items.h>
 
 GSList *windows; /* first in the list is the active window,
                     next is the last active, etc. */
@@ -679,10 +679,11 @@ static void sig_server_disconnected(SERVER_REC *server)
 	}
 }
 
-static void window_print_daychange(WINDOW_REC *window, struct tm *tm)
+static void window_print_daychange(WINDOW_REC *window, time_t t)
 {
         THEME_REC *theme;
         TEXT_DEST_REC dest;
+	struct tm *tm;
 	char *format, str[256];
 	int ret;
 
@@ -690,6 +691,7 @@ static void window_print_daychange(WINDOW_REC *window, struct tm *tm)
 	format_create_dest(&dest, NULL, NULL, MSGLEVEL_NEVER, window);
 	format = format_get_text_theme(theme, MODULE_NAME, &dest,
 				       TXT_DAYCHANGE);
+	tm = localtime(&t);
 	ret = strftime(str, sizeof(str), format, tm);
 	g_free(format);
 	if (ret <= 0) return;
@@ -770,7 +772,7 @@ static void sig_print_text(void)
 
 	/* day changed, print notice about it to every window */
 	for (tmp = windows; tmp != NULL; tmp = tmp->next)
-		window_print_daychange(tmp->data, tm);
+		window_print_daychange(tmp->data, t);
 }
 
 static int sig_check_daychange(void)
