@@ -39,9 +39,9 @@ FORMAT_REC fecommon_core_formats[] = {
 	{ "set_server_sticky", "Window's server set sticky", 1, { 0 } },
 	{ "unset_server_sticky", "Window's server isn't sticky anymore", 0 },
 	{ "window_name_not_unique", "Window names must be unique", 1, { 0 } },
-	{ "window_level", "Window level is now $0", 1, { 0 } },
-	{ "window_set_immortal", "Window is now immortal", 0 },
-	{ "window_unset_immortal", "Window isn't immortal anymore", 0 },
+	{ "window_level", "Window level is $0", 1, { 0 } },
+	{ "window_set_immortal", "Window is immortal", 0 },
+	{ "window_unset_immortal", "Window isn't immortal", 0 },
 	{ "window_immortal_error", "Window is immortal, if you really want to close it, say /WINDOW IMMORTAL OFF", 0 },
 	{ "windowlist_header", "%#Ref  Name                 Active item     Server          Level", 0 },
 	{ "windowlist_line", "%#$[4]0 %|$[20]1 $[15]2 $[15]3 $4", 5, { 1, 0, 0, 0, 0 } },
