@@ -19,23 +19,23 @@
 */
 
 #include "module.h"
-#include "module-formats.h"
-#include "signals.h"
-#include "misc.h"
-#include "settings.h"
-#include "special-vars.h"
-#include "levels.h"
-#include "servers.h"
+#include <irssi/src/fe-text/module-formats.h>
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/misc.h>
+#include <irssi/src/core/settings.h>
+#include <irssi/src/core/special-vars.h>
+#include <irssi/src/core/levels.h>
+#include <irssi/src/core/servers.h>
 
-#include "completion.h"
-#include "command-history.h"
-#include "keyboard.h"
-#include "printtext.h"
+#include <irssi/src/fe-common/core/completion.h>
+#include <irssi/src/fe-common/core/command-history.h>
+#include <irssi/src/fe-common/core/keyboard.h>
+#include <irssi/src/fe-common/core/printtext.h>
 
-#include "term.h"
-#include "gui-entry.h"
-#include "gui-windows.h"
-#include "utf8.h"
+#include <irssi/src/fe-text/term.h>
+#include <irssi/src/fe-text/gui-entry.h>
+#include <irssi/src/fe-text/gui-windows.h>
+#include <irssi/src/core/utf8.h>
 
 #include <string.h>
 #include <signal.h>
@@ -58,7 +58,7 @@ static int escape_next_key;
 
 static int readtag;
 static unichar prev_key;
-static GTimeVal last_keypress;
+static gint64 last_keypress;
 
 static int paste_detect_time, paste_verify_line_count;
 static char *paste_entry;
@@ -69,6 +69,7 @@ static GArray *paste_buffer_rest;
 static char *paste_old_prompt;
 static int paste_prompt, paste_line_count;
 static int paste_join_multiline;
+static int paste_ignore_first_nl;
 static int paste_timeout_id;
 static int paste_use_bracketed_mode;
 static int paste_bracketed_mode;
@@ -81,13 +82,32 @@ static int previous_yank_preceded;
 static const unichar bp_start[] = { 0x1b, '[', '2', '0', '0', '~' };
 static const unichar bp_end[]   = { 0x1b, '[', '2', '0', '1', '~' };
 
+#define BRACKETED_PASTE_TIMEOUT (5 * 1000) // ms
+
+#if GLIB_CHECK_VERSION(2, 62, 0)
+/* nothing */
+#else
+/* compatibility code for old GLib */
+GArray *g_array_copy(GArray *array)
+{
+	GArray *out;
+	guint elt_size;
+
+	elt_size = g_array_get_element_size(array);
+	out = g_array_sized_new(FALSE, FALSE, elt_size, array->len);
+	memcpy(out->data, array->data, array->len * elt_size);
+	out->len = array->len;
+
+	return out;
+}
+#endif
+
 static void sig_input(void);
 
 void input_listen_init(int handle)
 {
-	readtag = g_input_add_poll(handle,
-				   G_PRIORITY_HIGH, G_INPUT_READ,
-				   (GInputFunction) sig_input, NULL);
+	readtag = i_input_add_poll(handle, G_PRIORITY_HIGH, I_INPUT_READ,
+	                           (GInputFunction) sig_input, NULL);
 }
 
 void input_listen_deinit(void)
@@ -144,7 +164,7 @@ static int get_scroll_count(void)
 	else if (count < 1)
                 count = 1.0/count;
 
-	if (*str == '/') {
+	if (*str == '/' || *str == '.') {
 		count = (active_mainwin->height-active_mainwin->statusbar_lines)/count;
 	}
 	return (int)count;
@@ -331,7 +351,7 @@ static void paste_send(void)
 	g_string_free(str, TRUE);
 }
 
-static void paste_flush(int send)
+static void paste_flush(void (*send)(void))
 {
 	if (paste_prompt) {
 		gui_entry_set_text(active_entry, paste_entry);
@@ -339,8 +359,8 @@ static void paste_flush(int send)
 		g_free_and_null(paste_entry);
 	}
 
-	if (send)
-		paste_send();
+	if (send != NULL)
+		send();
 	g_array_set_size(paste_buffer, 0);
 
 	/* re-add anything that may have been after the bracketed paste end */
@@ -359,6 +379,128 @@ static void paste_flush(int send)
 	gui_entry_redraw(active_entry);
 }
 
+static void paste_print_line(const char *str)
+{
+	printformat_window(active_win, MSGLEVEL_CLIENTCRAP, TXT_PASTE_CONTENT, str);
+}
+
+static void paste_print(void)
+{
+	GArray *garr;
+	unichar *arr;
+	GString *str;
+	char out[10];
+	unsigned int i;
+	gboolean free_garr;
+
+	if (paste_join_multiline) {
+		garr = g_array_copy(paste_buffer);
+		paste_buffer_join_lines(garr);
+		free_garr = TRUE;
+	} else {
+		garr = paste_buffer;
+		free_garr = FALSE;
+	}
+
+	arr = &g_array_index(garr, unichar, 0);
+
+	str = g_string_new(NULL);
+	for (i = 0; i < garr->len; i++) {
+		if (isnewline(arr[i])) {
+			paste_print_line(str->str);
+			g_string_truncate(str, 0);
+		} else if (active_entry->utf8) {
+			out[g_unichar_to_utf8(arr[i], out)] = '\0';
+			g_string_append(str, out);
+		} else if (term_type == TERM_TYPE_BIG5) {
+			if (arr[i] > 0xff)
+				g_string_append_c(str, (arr[i] >> 8) & 0xff);
+			g_string_append_c(str, arr[i] & 0xff);
+		} else {
+			g_string_append_c(str, arr[i]);
+		}
+	}
+
+	if (str->len)
+		paste_print_line(str->str);
+
+	g_string_free(str, TRUE);
+	if (free_garr)
+		g_array_free(garr, TRUE);
+}
+
+static void paste_event(const char *arg)
+{
+	GArray *garr;
+	unichar *arr;
+	GString *str;
+	char out[10];
+	unsigned int i;
+	gboolean free_garr;
+
+	if (paste_join_multiline) {
+		garr = g_array_copy(paste_buffer);
+		paste_buffer_join_lines(garr);
+		free_garr = TRUE;
+	} else {
+		garr = paste_buffer;
+		free_garr = FALSE;
+	}
+
+	arr = &g_array_index(garr, unichar, 0);
+	str = g_string_new(NULL);
+	for (i = 0; i < garr->len; i++) {
+		if (isnewline(arr[i])) {
+			g_string_append_c(str, '\n');
+		} else if (active_entry->utf8) {
+			out[g_unichar_to_utf8(arr[i], out)] = '\0';
+			g_string_append(str, out);
+		} else if (term_type == TERM_TYPE_BIG5) {
+			if (arr[i] > 0xff)
+				g_string_append_c(str, (arr[i] >> 8) & 0xff);
+			g_string_append_c(str, arr[i] & 0xff);
+		} else {
+			g_string_append_c(str, arr[i]);
+		}
+	}
+
+	if (signal_emit("paste event", 2, str->str, arg)) {
+		paste_flush(NULL);
+	}
+
+	g_string_free(str, TRUE);
+	if (free_garr)
+		g_array_free(garr, TRUE);
+}
+
+static void paste_insert_edit(void)
+{
+	unichar *arr;
+	unsigned int i;
+
+	if (paste_join_multiline)
+		paste_buffer_join_lines(paste_buffer);
+
+	arr = &g_array_index(paste_buffer, unichar, 0);
+	for (i = 0; i < paste_buffer->len; i++) {
+		if (isnewline(arr[i])) {
+			gui_entry_insert_char(active_entry, '\\');
+			gui_entry_insert_char(active_entry, 'n');
+		} else if (arr[i] == 9) {
+			gui_entry_insert_char(active_entry, '\\');
+			gui_entry_insert_char(active_entry, 't');
+		} else if (arr[i] == 27) {
+			gui_entry_insert_char(active_entry, '\\');
+			gui_entry_insert_char(active_entry, 'e');
+		} else if (arr[i] == '\\') {
+			gui_entry_insert_char(active_entry, '\\');
+			gui_entry_insert_char(active_entry, '\\');
+		} else {
+			gui_entry_insert_char(active_entry, arr[i]);
+		}
+	}
+}
+
 static void insert_paste_prompt(void)
 {
 	char *str;
@@ -395,8 +537,8 @@ static void insert_paste_prompt(void)
 
 static void sig_gui_key_pressed(gpointer keyp)
 {
-	GTimeVal now;
-        unichar key;
+	gint64 now;
+	unichar key;
 	char str[20];
 	int ret;
 
@@ -407,7 +549,7 @@ static void sig_gui_key_pressed(gpointer keyp)
 		return;
 	}
 
-        g_get_current_time(&now);
+	now = g_get_real_time();
 
 	if (key < 32) {
 		/* control key */
@@ -730,6 +872,17 @@ static gboolean paste_timeout(gpointer data)
 	int split_lines;
 	paste_was_bracketed_mode = paste_bracketed_mode;
 
+	if (paste_ignore_first_nl && paste_line_count == 1) {
+		unichar last_char;
+
+		last_char = g_array_index(paste_buffer, unichar, paste_buffer->len - 1);
+
+		if (isnewline(last_char)) {
+			g_array_set_size(paste_buffer, paste_buffer->len - 1);
+			paste_line_count--;
+		}
+	}
+
 	/* number of lines after splitting extra-long messages */
 	split_lines = paste_buffer->len / LINE_SPLIT_LIMIT;
 
@@ -748,7 +901,7 @@ static gboolean paste_timeout(gpointer data)
 				active_win->active != NULL)
 		insert_paste_prompt();
 	else
-		paste_flush(TRUE);
+		paste_flush(paste_send);
 	paste_timeout_id = -1;
 	return FALSE;
 }
@@ -778,12 +931,14 @@ static void paste_bracketed_end(int i, gboolean rest)
 	}
 
 	/* decide what to do with the buffer */
+	if (paste_timeout_id != -1)
+		g_source_remove(paste_timeout_id);
 	paste_timeout(NULL);
 
 	paste_bracketed_mode = FALSE;
 }
 
-static void paste_bracketed_middle()
+static void paste_bracketed_middle(void)
 {
 	int i;
 	int marklen = G_N_ELEMENTS(bp_end);
@@ -831,8 +986,10 @@ static void sig_input(void)
 		term_gets(buffer, &line_count);
 		key = g_array_index(buffer, unichar, 0);
 		/* Either Ctrl-k or Ctrl-c is pressed */
-		if (key == 11 || key == 3)
-			paste_flush(key == 11);
+		if (key < 32 && key != 13 /* CR */ && key != 10 /* LF */ && key != 27 /* Esc */) {
+			key_pressed(keyboard, "paste");
+			signal_emit("gui key pressed", 1, GINT_TO_POINTER(key));
+		}
 		g_array_free(buffer, TRUE);
 	} else {
 		term_gets(paste_buffer, &paste_line_count);
@@ -870,12 +1027,52 @@ static void sig_input(void)
 
 static void key_paste_start(void)
 {
-	paste_bracketed_mode = TRUE;
+	if (paste_use_bracketed_mode) {
+		paste_bracketed_mode = TRUE;
+		if (paste_timeout_id != -1)
+			g_source_remove(paste_timeout_id);
+		paste_timeout_id = g_timeout_add(BRACKETED_PASTE_TIMEOUT, paste_timeout, NULL);
+	}
+}
+
+static void key_paste_cancel(void)
+{
+	if (paste_prompt) {
+		paste_flush(NULL);
+	}
+}
+
+static void key_paste_print(void)
+{
+	if (paste_prompt) {
+		paste_print();
+	}
+}
+
+static void key_paste_send(void)
+{
+	if (paste_prompt) {
+		paste_flush(paste_send);
+	}
+}
+
+static void key_paste_edit(void)
+{
+	if (paste_prompt) {
+		paste_flush(paste_insert_edit);
+	}
+}
+
+static void key_paste_event(const char *arg)
+{
+	if (paste_prompt) {
+		paste_event(arg);
+	}
 }
 
 time_t get_idle_time(void)
 {
-	return last_keypress.tv_sec;
+	return last_keypress / G_TIME_SPAN_SECOND;
 }
 
 static void key_scroll_backward(void)
@@ -1132,11 +1329,14 @@ static void setup_changed(void)
 
 	paste_verify_line_count = settings_get_int("paste_verify_line_count");
 	paste_join_multiline = settings_get_bool("paste_join_multiline");
+	paste_ignore_first_nl = settings_get_bool("paste_ignore_first_nl");
 	paste_use_bracketed_mode = settings_get_bool("paste_use_bracketed_mode");
 
 	term_set_appkey_mode(settings_get_bool("term_appkey_mode"));
 	/* Enable the bracketed paste mode on demand */
 	term_set_bracketed_paste_mode(paste_use_bracketed_mode);
+	if (!paste_use_bracketed_mode)
+		paste_bracketed_mode = FALSE;
 }
 
 void gui_readline_init(void)
@@ -1154,8 +1354,8 @@ void gui_readline_init(void)
         paste_old_prompt = NULL;
 	paste_timeout_id = -1;
 	paste_bracketed_mode = FALSE;
-	g_get_current_time(&last_keypress);
-        input_listen_init(STDIN_FILENO);
+	last_keypress = g_get_real_time();
+	input_listen_init(STDIN_FILENO);
 
 	settings_add_bool("lookandfeel", "term_appkey_mode", TRUE);
 	settings_add_str("history", "scroll_page_count", "/2");
@@ -1165,7 +1365,8 @@ void gui_readline_init(void)
 	   keycodes. this must be larger to allow them to work. */
 	settings_add_int("misc", "paste_verify_line_count", 5);
 	settings_add_bool("misc", "paste_join_multiline", TRUE);
-        setup_changed();
+	settings_add_bool("misc", "paste_ignore_first_nl", FALSE);
+	setup_changed();
 
 	keyboard = keyboard_create(NULL);
         key_configure_freeze();
@@ -1236,7 +1437,13 @@ void gui_readline_init(void)
 
 	key_bind("key", NULL, "meta-O-M", "return", (SIGNAL_FUNC) key_combo);
 
-	key_bind("paste_start", "Bracketed paste start", "meta2-200~", "paste_start", (SIGNAL_FUNC) key_paste_start);
+	/* clang-format off */
+	key_bind("paste_start", "Bracketed paste start", "^[[200~", "paste_start", (SIGNAL_FUNC) key_paste_start);
+	key_bind("paste_cancel", "Cancel paste", "paste-^C", NULL, (SIGNAL_FUNC) key_paste_cancel);
+	key_bind("paste_print", "Print paste to screen", "paste-^P", NULL, (SIGNAL_FUNC) key_paste_print);
+	key_bind("paste_send", "Send paste to target", "paste-^K", NULL, (SIGNAL_FUNC) key_paste_send);
+	key_bind("paste_edit", "Insert paste to input line", "paste-^E", NULL, (SIGNAL_FUNC) key_paste_edit);
+	key_bind("paste_event", "Send paste to event", "paste-^U", NULL, (SIGNAL_FUNC) key_paste_event);
 
 	/* cursor movement */
 	key_bind("backward_character", "Move the cursor a character backward", "left", NULL, (SIGNAL_FUNC) key_backward_character);
@@ -1308,8 +1515,9 @@ void gui_readline_init(void)
         /* inserting special input characters to line.. */
 	key_bind("escape_char", "Insert the next character exactly as-is to input line", NULL, NULL, (SIGNAL_FUNC) key_escape);
 	key_bind("insert_text", "Append text to line", NULL, NULL, (SIGNAL_FUNC) key_insert_text);
+	/* clang-format on */
 
-        /* autoreplaces */
+	/* autoreplaces */
 	key_bind("multi", NULL, "return", "check_replaces;send_line", NULL);
 	key_bind("multi", NULL, "space", "check_replaces;insert_text  ", NULL);
 
@@ -1339,6 +1547,11 @@ void gui_readline_deinit(void)
         key_configure_freeze();
 
 	key_unbind("paste_start", (SIGNAL_FUNC) key_paste_start);
+	key_unbind("paste_cancel", (SIGNAL_FUNC) key_paste_cancel);
+	key_unbind("paste_print", (SIGNAL_FUNC) key_paste_print);
+	key_unbind("paste_send", (SIGNAL_FUNC) key_paste_send);
+	key_unbind("paste_edit", (SIGNAL_FUNC) key_paste_edit);
+	key_unbind("paste_event", (SIGNAL_FUNC) key_paste_event);
 
 	key_unbind("backward_character", (SIGNAL_FUNC) key_backward_character);
 	key_unbind("forward_character", (SIGNAL_FUNC) key_forward_character);
