@@ -1,8 +1,8 @@
-#ifndef __GUI_WINDOWS_H
-#define __GUI_WINDOWS_H
+#ifndef IRSSI_FE_TEXT_GUI_WINDOWS_H
+#define IRSSI_FE_TEXT_GUI_WINDOWS_H
 
-#include "mainwindows.h"
-#include "textbuffer-view.h"
+#include <irssi/src/fe-text/mainwindows.h>
+#include <irssi/src/fe-text/textbuffer-view.h>
 
 #define WINDOW_GUI(a) ((GUI_WINDOW_REC *) ((a)->gui_data))
 #define WINDOW_MAIN(a) (WINDOW_GUI(a)->parent)
