@@ -19,29 +19,29 @@
 */
 
 #include "module.h"
-#include "module-formats.h"
-#include "modules-load.h"
-#include "args.h"
-#include "signals.h"
-#include "levels.h"
-#include "core.h"
-#include "settings.h"
-#include "session.h"
-#include "servers.h"
+#include <irssi/src/fe-text/module-formats.h>
+#include <irssi/src/core/modules-load.h>
+#include <irssi/src/core/args.h>
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/levels.h>
+#include <irssi/src/core/core.h>
+#include <irssi/src/core/settings.h>
+#include <irssi/src/core/session.h>
+#include <irssi/src/core/servers.h>
 
-#include "printtext.h"
-#include "fe-common-core.h"
-#include "fe-settings.h"
-#include "themes.h"
+#include <irssi/src/fe-common/core/printtext.h>
+#include <irssi/src/fe-common/core/fe-common-core.h>
+#include <irssi/src/fe-common/core/fe-settings.h>
+#include <irssi/src/fe-common/core/themes.h>
 
-#include "term.h"
-#include "gui-entry.h"
-#include "mainwindows.h"
-#include "gui-printtext.h"
-#include "gui-readline.h"
-#include "statusbar.h"
-#include "gui-windows.h"
-#include "irssi-version.h"
+#include <irssi/src/fe-text/term.h>
+#include <irssi/src/fe-text/gui-entry.h>
+#include <irssi/src/fe-text/mainwindows.h>
+#include <irssi/src/fe-text/gui-printtext.h>
+#include <irssi/src/fe-text/gui-readline.h>
+#include <irssi/src/fe-text/statusbar.h>
+#include <irssi/src/fe-text/gui-windows.h>
+#include <irssi/irssi-version.h>
 
 #include <signal.h>
 #include <locale.h>
@@ -71,6 +71,9 @@ void gui_expandos_deinit(void);
 void textbuffer_commands_init(void);
 void textbuffer_commands_deinit(void);
 
+void textbuffer_formats_init(void);
+void textbuffer_formats_deinit(void);
+
 void lastlog_init(void);
 void lastlog_deinit(void);
 
@@ -99,6 +102,21 @@ static void sig_settings_userinfo_changed(gpointer changedp)
 	user_settings_changed = GPOINTER_TO_UINT(changedp);
 }
 
+static void sig_autoload_modules(void)
+{
+	char **list, **module;
+	list = g_strsplit_set(settings_get_str("autoload_modules"), " ,", -1);
+	for (module = list; *module != NULL; module++) {
+		char *tmp;
+		if ((tmp = strchr(*module, ':')) != NULL)
+			*tmp = ' ';
+		tmp = g_strdup_printf("-silent %s", *module);
+		signal_emit("command load", 1, tmp);
+		g_free(tmp);
+	}
+	g_strfreev(list);
+}
+
 /* redraw irssi's screen.. */
 void irssi_redraw(void)
 {
@@ -156,9 +174,20 @@ static void textui_init(void)
 
 	theme_register(gui_text_formats);
 	signal_add("settings userinfo changed", (SIGNAL_FUNC) sig_settings_userinfo_changed);
+	signal_add("module autoload", (SIGNAL_FUNC) sig_autoload_modules);
 	signal_add_last("gui exit", (SIGNAL_FUNC) sig_exit);
 }
 
+static int critical_fatal_section_begin(void)
+{
+	return g_log_set_always_fatal(G_LOG_FATAL_MASK | G_LOG_LEVEL_CRITICAL);
+}
+
+static void critical_fatal_section_end(int loglev)
+{
+	g_log_set_always_fatal(loglev);
+}
+
 static void textui_finish_init(void)
 {
 	int loglev;
@@ -168,6 +197,7 @@ static void textui_finish_init(void)
 	textbuffer_init();
 	textbuffer_view_init();
 	textbuffer_commands_init();
+	textbuffer_formats_init();
 	gui_expandos_init();
 	gui_printtext_init();
 	gui_readline_init();
@@ -178,10 +208,9 @@ static void textui_finish_init(void)
 	mainwindows_layout_init();
 	gui_windows_init();
 	/* Temporarily raise the fatal level to abort on config errors. */
-	loglev = g_log_set_always_fatal(G_LOG_FATAL_MASK | G_LOG_LEVEL_CRITICAL);
+	loglev = critical_fatal_section_begin();
 	statusbar_init();
-	g_log_set_always_fatal(loglev);
-	term_refresh_thaw();
+	critical_fatal_section_end(loglev);
 
 	settings_check();
 
@@ -198,7 +227,12 @@ static void textui_finish_init(void)
 
 	dirty_check();
 
+	/* Temporarily raise the fatal level to abort on config errors. */
+	loglev = critical_fatal_section_begin();
 	fe_common_core_finish_init();
+	critical_fatal_section_end(loglev);
+	term_refresh_thaw();
+
 	signal_emit("irssi init finished", 0);
 	statusbar_redraw(NULL, TRUE);
 
@@ -244,6 +278,7 @@ static void textui_deinit(void)
 
 	dirty_check(); /* one last time to print any quit messages */
 	signal_remove("settings userinfo changed", (SIGNAL_FUNC) sig_settings_userinfo_changed);
+	signal_remove("module autoload", (SIGNAL_FUNC) sig_autoload_modules);
 	signal_remove("gui exit", (SIGNAL_FUNC) sig_exit);
 
 	lastlog_deinit();
@@ -256,6 +291,7 @@ static void textui_deinit(void)
 	mainwindow_activity_deinit();
 	mainwindows_deinit();
 	gui_expandos_deinit();
+	textbuffer_formats_deinit();
 	textbuffer_commands_deinit();
 	textbuffer_view_deinit();
 	textbuffer_deinit();
@@ -324,7 +360,7 @@ int main(int argc, char **argv)
 	setlocale(LC_ALL, "");
 
 	/* Temporarily raise the fatal level to abort on config errors. */
-	loglev = g_log_set_always_fatal(G_LOG_FATAL_MASK | G_LOG_LEVEL_CRITICAL);
+	loglev = critical_fatal_section_begin();
 	textui_init();
 
 	if (!term_init()) {
@@ -332,27 +368,33 @@ int main(int argc, char **argv)
 		return 1;
 	}
 
-	g_log_set_always_fatal(loglev);
+	critical_fatal_section_end(loglev);
+
 	textui_finish_init();
-	main_loop = g_main_new(TRUE);
+	main_loop = g_main_loop_new(NULL, TRUE);
 
 	/* Does the same as g_main_run(main_loop), except we
 	   can call our dirty-checker after each iteration */
 	while (!quitting) {
-		if (reload_config) {
-			/* SIGHUP received, do /RELOAD */
-			reload_config = FALSE;
-			signal_emit("command reload", 1, "");
+		if (sighup_received) {
+			sighup_received = FALSE;
+
+			if (settings_get_bool("quit_on_hup")) {
+				signal_emit("gui exit", 0);
+			} 
+			else {
+				signal_emit("command reload", 1, "");
+			}
 		}
 
 		dirty_check();
 
 		term_refresh_freeze();
-		g_main_iteration(TRUE);
+		g_main_context_iteration(NULL, TRUE);
 		term_refresh_thaw();
 	}
 
-	g_main_destroy(main_loop);
+	g_main_loop_unref(main_loop);
 	textui_deinit();
 
 	session_upgrade(); /* if we /UPGRADEd, start the new process */
