@@ -19,10 +19,10 @@
 */
 
 #include "module.h"
-#include "signals.h"
-#include "settings.h"
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/settings.h>
 
-#include "gui-windows.h"
+#include <irssi/src/fe-text/gui-windows.h>
 
 /* Don't send window activity if window is already visible in
    another mainwindow */
