@@ -19,22 +19,22 @@
 */
 
 #include "module.h"
-#include "misc.h"
-#include "recode.h"
-#include "special-vars.h"
-#include "settings.h"
-#include "window-item-def.h"
+#include <irssi/src/core/misc.h>
+#include <irssi/src/core/recode.h>
+#include <irssi/src/core/special-vars.h>
+#include <irssi/src/core/settings.h>
+#include <irssi/src/core/window-item-def.h>
 
-#include "servers-reconnect.h"
-#include "servers-redirect.h"
-#include "servers-setup.h"
-#include "nicklist.h"
+#include <irssi/src/core/servers-reconnect.h>
+#include <irssi/src/irc/core/servers-redirect.h>
+#include <irssi/src/core/servers-setup.h>
+#include <irssi/src/core/nicklist.h>
 
-#include "bans.h"
-#include "irc-commands.h"
-#include "irc-servers.h"
-#include "irc-channels.h"
-#include "irc-queries.h"
+#include <irssi/src/irc/core/bans.h>
+#include <irssi/src/irc/core/irc-commands.h>
+#include <irssi/src/irc/core/irc-servers.h>
+#include <irssi/src/irc/core/irc-channels.h>
+#include <irssi/src/irc/core/irc-queries.h>
 
 /* How often to check if there's anyone to be unbanned in knockout list */
 #define KNOCKOUT_TIMECHECK 10000
@@ -438,15 +438,18 @@ static void cmd_whois(const char *data, IRC_SERVER_REC *server,
 	query = get_redirect_nicklist(query, &free_nick);
 
 	str = g_strconcat(qserver, " ", query, NULL);
-	server_redirect_event(server, "whois", 1, str, TRUE,
-	              NULL,
-	              "event 318", "whois end",
-	              "event 402", event_402,
-	              "event 301", "whois away", /* 301 can come as a reply to /MSG, /WHOIS or /WHOWAS */
-	              "event 313", "whois oper",
-	              "event 401", (settings_get_bool("auto_whowas") ? "whois try whowas" : "whois event not found"),
-	              "event 311", "whois event",
-	              "", "whois default event", NULL);
+	server_redirect_event(
+	    server, "whois", 1, str, TRUE, /* */
+	    NULL,                          /* */
+	    "event 318", "whois end",      /* */
+	    "event 402", event_402,        /* */
+	    "event 301", "whois away",     /* 301 can come as a reply to /MSG, /WHOIS or /WHOWAS */
+	    "event 313", "whois oper",     /* */
+	    "event 330", "whois account",  /* */
+	    "event 401",
+	    (settings_get_bool("auto_whowas") ? "whois try whowas" : "whois event not found"),
+	    "event 311", "whois event", /* */
+	    "", "whois default event", NULL);
 	g_free(str);
 
 	server->whois_found = FALSE;
@@ -528,7 +531,7 @@ static void cmd_whowas(const char *data, IRC_SERVER_REC *server)
 /* SYNTAX: PING [<nick> | <channel> | *] */
 static void cmd_ping(const char *data, IRC_SERVER_REC *server, WI_ITEM_REC *item)
 {
-	GTimeVal tv;
+	gint64 tv;
 	char *str;
 
 	CMD_IRC_SERVER(server);
@@ -539,9 +542,12 @@ static void cmd_ping(const char *data, IRC_SERVER_REC *server, WI_ITEM_REC *item
 		data = window_item_get_target(item);
 	}
 
-	g_get_current_time(&tv);
+	tv = g_get_real_time();
 
-	str = g_strdup_printf("%s PING %ld %ld", data, tv.tv_sec, tv.tv_usec);
+	str = g_strdup_printf("%s PING "
+	                      "%" G_GINT64_FORMAT " "
+	                      "%" G_GINT64_FORMAT,
+	                      data, tv / G_TIME_SPAN_SECOND, tv % G_TIME_SPAN_SECOND);
 	signal_emit("command ctcp", 3, str, server, item);
 	g_free(str);
 }
@@ -616,13 +622,8 @@ static void cmd_wait(const char *data, IRC_SERVER_REC *server)
 
 	n = atoi(msecs);
 	if (server != NULL && n > 0) {
-		g_get_current_time(&server->wait_cmd);
-		server->wait_cmd.tv_sec += n/1000;
-		server->wait_cmd.tv_usec += n%1000;
-		if (server->wait_cmd.tv_usec >= 1000) {
-			server->wait_cmd.tv_sec++;
-			server->wait_cmd.tv_usec -= 1000;
-		}
+		server->wait_cmd = g_get_real_time();
+		server->wait_cmd += n * G_TIME_SPAN_MILLISECOND;
 	}
 	cmd_params_free(free_arg);
 }
@@ -1045,6 +1046,7 @@ void irc_commands_init(void)
 	command_bind_irc("wallops", NULL, (SIGNAL_FUNC) command_1self);
 	command_bind_irc("kickban", NULL, (SIGNAL_FUNC) cmd_kickban);
 	command_bind_irc("knockout", NULL, (SIGNAL_FUNC) cmd_knockout);
+	command_bind_irc("setname", NULL, (SIGNAL_FUNC) command_1self);
 	command_bind_irc("server purge", NULL, (SIGNAL_FUNC) cmd_server_purge);
 
 	signal_add("channel destroyed", (SIGNAL_FUNC) sig_channel_destroyed);
@@ -1054,7 +1056,7 @@ void irc_commands_init(void)
 	signal_add("whois end", (SIGNAL_FUNC) event_end_of_whois);
 	signal_add("whowas event", (SIGNAL_FUNC) event_whowas);
 
-	command_set_options("connect", "+ircnet");
+	command_set_options("connect", "+ircnet starttls disallow_starttls nocap");
 	command_set_options("topic", "delete");
 	command_set_options("list", "yes");
 	command_set_options("away", "one all");
@@ -1111,6 +1113,7 @@ void irc_commands_deinit(void)
 	command_unbind("wait", (SIGNAL_FUNC) cmd_wait);
 	command_unbind("wallops", (SIGNAL_FUNC) command_1self);
 	command_unbind("kickban", (SIGNAL_FUNC) cmd_kickban);
+	command_unbind("setname", (SIGNAL_FUNC) command_1self);
 	command_unbind("knockout", (SIGNAL_FUNC) cmd_knockout);
 	command_unbind("server purge", (SIGNAL_FUNC) cmd_server_purge);
 
