@@ -19,12 +19,12 @@
 */
 
 #include "module.h"
-#include "signals.h"
-#include "misc.h"
-#include "rawlog.h"
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/misc.h>
+#include <irssi/src/core/rawlog.h>
 
-#include "irc-servers.h"
-#include "servers-redirect.h"
+#include <irssi/src/irc/core/irc-servers.h>
+#include <irssi/src/irc/core/servers-redirect.h>
 
 #define DEFAULT_REDIRECT_TIMEOUT 60
 
@@ -39,6 +39,7 @@ typedef struct {
 
 	int remote;
 	int timeout;
+	int pos;
 	GSList *start, *stop, *opt; /* char *event, int argpos, ... */
 } REDIRECT_CMD_REC;
 
@@ -151,13 +152,11 @@ void server_redirect_register(const char *command,
 
 	va_end(va);
 
-	server_redirect_register_list(command, remote, timeout,
-				      start, stop, opt);
+	server_redirect_register_list(command, remote, timeout, start, stop, opt, 0);
 }
 
-void server_redirect_register_list(const char *command,
-				   int remote, int timeout,
-				   GSList *start, GSList *stop, GSList *opt)
+void server_redirect_register_list(const char *command, int remote, int timeout, GSList *start,
+                                   GSList *stop, GSList *opt, int pos)
 {
 	REDIRECT_CMD_REC *rec;
         gpointer key, value;
@@ -181,7 +180,8 @@ void server_redirect_register_list(const char *command,
 	rec->start = start;
         rec->stop = stop;
         rec->opt = opt;
-        g_hash_table_insert(command_redirects, rec->name, rec);
+	rec->pos = pos;
+	g_hash_table_insert(command_redirects, rec->name, rec);
 }
 
 void server_redirect_event(IRC_SERVER_REC *server, const char *command,
@@ -219,7 +219,7 @@ static char *signal_list_move(GSList **signals, const char *event)
 	GSList *link;
         char *linkevent, *linksignal;
 
-	link = gslist_find_string(*signals, event);
+	link = i_slist_find_string(*signals, event);
 	if (link == NULL)
 		return NULL;
 
@@ -442,7 +442,7 @@ static void redirect_abort(IRC_SERVER_REC *server, REDIRECT_REC *rec)
 		g_free(str);
 
 		if (rec->failure_signal != NULL)
-			signal_emit(rec->failure_signal, 1, server);
+			signal_emit(rec->failure_signal, 3, server, rec->cmd->name, rec->arg);
 	} else if (rec->last_signal != NULL) {
                 /* emit the last signal */
 		signal_emit(rec->last_signal, 1, server);
@@ -483,6 +483,32 @@ static REDIRECT_REC *redirect_find(IRC_SERVER_REC *server, const char *event,
 		}
 	}
 
+	if (g_strcmp0("event 263", event) == 0) { /* RPL_TRYAGAIN */
+		char *params, *command;
+		params = event_get_params(args, 3, NULL, &command, NULL);
+
+		for (tmp = server->redirects; tmp != NULL; tmp = next) {
+			REDIRECT_REC *rec = tmp->data;
+
+			next = tmp->next;
+
+			if (rec == redirect)
+				break;
+
+			if (g_slist_find(server->redirect_active, rec) != NULL)
+				continue;
+
+			if (redirect_args_match(rec->cmd->name, command, rec->cmd->pos)) {
+				/* the server crashed our command with RPL_TRYAGAIN, send the
+				   failure */
+				rec->aborted = TRUE;
+				redirect_abort(server, rec);
+				break;
+			}
+		}
+		g_free(params);
+	}
+
 	/* remove the destroyed, non-remote and timeouted remote
 	   redirections that should have happened before this redirection */
 	now = time(NULL);
@@ -671,7 +697,16 @@ void servers_redirect_init(void)
 				 NULL,
 				 NULL);
 
-        /* LIST */
+	/* WHO user */
+	server_redirect_register("who user", FALSE, 0, /* */
+	                         "event 352", 5,       /* An element of the WHO */
+	                         "event 354", -1,      /* WHOX element */
+	                         NULL,                 /* */
+	                         "event 315", 1,       /* End of WHO */
+	                         NULL,                 /* */
+	                         NULL);
+
+	/* LIST */
 	server_redirect_register("list", FALSE, 0,
 				 "event 321", 1, /* Begins the LIST */
 				 NULL,
