@@ -19,15 +19,15 @@
 */
 
 #include "module.h"
-#include "signals.h"
-#include "misc.h"
-#include "settings.h"
+#include <irssi/src/core/signals.h>
+#include <irssi/src/core/misc.h>
+#include <irssi/src/core/settings.h>
 
-#include "irc.h"
-#include "irc-servers.h"
-#include "servers-redirect.h"
+#include <irssi/src/irc/core/irc.h>
+#include <irssi/src/irc/core/irc-servers.h>
+#include <irssi/src/irc/core/servers-redirect.h>
 
-#include "notifylist.h"
+#include <irssi/src/irc/notifylist/notifylist.h>
 
 #define DEFAULT_NOTIFY_CHECK_TIME "1min"
 #define DEFAULT_NOTIFY_WHOIS_TIME "5min"
@@ -92,7 +92,7 @@ static void ison_send(IRC_SERVER_REC *server, GString *cmd)
 
 	server_redirect_event(server, "ison", 1, NULL, -1, NULL,
 			      "event 303", "notifylist event", NULL);
-	irc_send_cmd(server, cmd->str);
+	irc_send_cmd_later(server, cmd->str);
 
 	g_string_truncate(cmd, 0);
 }
@@ -131,7 +131,7 @@ static void notifylist_timeout_server(IRC_SERVER_REC *server)
 
 		len = strlen(nick);
 
-		if (cmd->len+len+1 > 510)
+		if (cmd->len+len+1 > server->max_message_len)
                         ison_send(server, cmd);
 
 		g_string_append_printf(cmd, "%s ", nick);
@@ -183,7 +183,9 @@ static void whois_send(IRC_SERVER_REC *server, const char *nicks,
 			      "", "event empty", NULL);
 	g_free(str);
 
-	irc_send_cmdv(server, "WHOIS %s", whois_request);
+	str = g_strdup_printf("WHOIS %s", whois_request);
+	irc_send_cmd_later(server, str);
+	g_free(str);
 }
 
 static void whois_send_server(IRC_SERVER_REC *server, char *nick)
@@ -279,7 +281,7 @@ static void ison_check_parts(IRC_SERVER_REC *server)
 		NOTIFY_NICK_REC *rec = tmp->data;
 		next = tmp->next;
 
-		if (gslist_find_icase_string(mserver->ison_tempusers, rec->nick) != NULL)
+		if (i_slist_find_icase_string(mserver->ison_tempusers, rec->nick) != NULL)
 			continue;
 
                 notifylist_left(server, rec);
