@@ -24,15 +24,15 @@
 #include <gcrypt.h>
 #include <unistd.h>
 
-#include "common.h"
-#include "levels.h"
-#include "signals.h"
-#include "printtext.h"
-#include "statusbar-item.h"
+#include <irssi/src/common.h>
+#include <irssi/src/core/levels.h>
+#include <irssi/src/core/signals.h>
+#include <irssi/src/fe-common/core/printtext.h>
+#include <irssi/src/fe-text/statusbar-item.h>
 
-#include "irssi-otr.h"
-#include "otr-formats.h"
-#include "key.h"
+#include <irssi/src/otr/irssi-otr.h>
+#include <irssi/src/otr/otr-formats.h>
+#include <irssi/src/otr/key.h>
 
 static int otr_debug = 0;
 
@@ -643,7 +643,7 @@ static enum otr_msg_status enqueue_otr_fragment(const char *msg, struct otr_peer
 		 * Dup the string with enough space for the NULL byte since we are
 		 * about to free it before passing it to the caller.
 		 */
-		*full_msg = strndup(opc->full_msg, opc->msg_len + 1);
+		*full_msg = g_strndup(opc->full_msg, opc->msg_len + 1);
 		/* Reset everything. */
 		free(opc->full_msg);
 		opc->full_msg = NULL;
