@@ -1,8 +1,8 @@
-#ifndef __PERL_SIGNALS_H
-#define __PERL_SIGNALS_H
+#ifndef IRSSI_PERL_PERL_SIGNALS_H
+#define IRSSI_PERL_PERL_SIGNALS_H
 
-void perl_signal_args_to_c(void (*callback)(void *, void **), void *cb_arg,
-                           int signal_id, SV **args, size_t n_args);
+void perl_signal_args_to_c(void (*callback)(void *, int, void **), void *cb_arg, int signal_id,
+                           SV **args, size_t n_args);
 
 void perl_signal_add_full(const char *signal, SV *func, int priority);
 
@@ -24,6 +24,8 @@ void perl_command_unbind(const char *cmd, SV *func);
 void perl_command_runsub(const char *cmd, const char *data,
 			 SERVER_REC *server, WI_ITEM_REC *item);
 
+void irssi_add_signal_arg_conv(const char *stash, PERL_BLESS_FUNC func);
+
 void perl_signal_register(const char *signal, const char **args);
 
 void perl_signals_start(void);
