@@ -1,5 +1,5 @@
-#include "common.h"
-#include "gui-readline.c"
+#include <irssi/src/common.h>
+#include <irssi/src/fe-text/gui-readline.c>
 
 typedef struct {
 	char const *const description;
